/* 
 *  $Id: ChannelWatchdog.java,v 1.1.1.1 2000/03/13 21:26:49 andrew Exp $
 *  
 *  Copyright 1996 Massachusetts Institute of Technology
 * 
 *  Permission to use, copy, modify, distribute, and sell this software and
 *  its documentation for any purpose is hereby granted without fee, provided
 *  that the above copyright notice appear in all copies and that both that
 *  copyright notice and this permission notice appear in supporting
 *  documentation, and that the name of M.I.T. not be used in advertising or
 *  publicity pertaining to distribution of the software without specific,
 *  written prior permission.  M.I.T. makes no representations about the
 *  suitability of this software for any purpose.  It is provided "as is"
 *  without express or implied warranty.
 * 
 */


package ants;


/** 
 * Thread of control that watches the channel thread to ensure that
 * it is making progress, and otherwise takes action to interrupt
 * capsule processing
 *
 * @author David Wetherall
 */


public class ChannelWatchdog
  extends Thread
{

  // channel on behalf of which we live
  // set and fixed in constructor
  Channel source;
  
  // the last checkpoint used to assess progress
  private Capsule previous;

  public void run() {

    while (true) {
      try {

	// let the channel run for a time
	Thread.sleep(1000);

	// what are we up to now?
	Capsule current = source.client.context;

	// ignore if we are not capsule processing or its not this channel
	if ((current == null) || (current.chan != source)) continue;

	// otherwise, if we have not made progress then interrupt
	if (current == previous) {
	  source.getThread().stop(new TimeLimitException());
	  Entity.info("ChannelWatchdog fired for capsule "+ current);
	}

	// update for next time around
	previous = current;
	
      } catch (Exception e) {
	if (Entity.LOGGING) Entity.error("ChannelWatchdog raises "+ e);
      }
    }      
  }
  
  public ChannelWatchdog(Channel c) {
    super();
    source = c;
  }
}


