/* 
 *  $Id: ConfigurationManager.java,v 1.3 2000/03/31 03:42:03 andrew Exp $
 *  
 *  Copyright 1996 Massachusetts Institute of Technology
 * 
 *  Permission to use, copy, modify, distribute, and sell this software and
 *  its documentation for any purpose is hereby granted without fee, provided
 *  that the above copyright notice appear in all copies and that both that
 *  copyright notice and this permission notice appear in supporting
 *  documentation, and that the name of M.I.T. not be used in advertising or
 *  publicity pertaining to distribution of the software without specific,
 *  written prior permission.  M.I.T. makes no representations about the
 *  suitability of this software for any purpose.  It is provided "as is"
 *  without express or implied warranty.
 * 
 */


package ants;

import java.awt.*;

import java.io.*;
import java.net.*;

import utils.*;


/**
 * manager that creates node/channel/app configurations
 * as specified in a control file
 *
 * @author David Wetherall
 */


public class ConfigurationManager extends Manager {

  /** This variable controls whether ANTS is running over anetd */
  public static final boolean USE_ANETD = true;
   
  public static final String [] defaults = {};
  public void setArgs(KeyArgs k) throws Exception {
    k.merge(defaults);
    
    for (int i = 0; i < k.length(); i++) {
      if (k.key(i).equals("-gui")) {
	k.strike(i); // This is no longer used 
      }
    }
    
    super.setArgs(k);
  }
 
  public ConfigurationManager(String[] params) throws Exception {
    super();

    String file = params[0];
    String node = params[1];

    KeyArgs k = new KeyArgs(params, 2);
    Node n = null;

    // process the configuration file, creating entities

    LineListTokenizer lt = new LineListTokenizer(file);
    String[] args;
    while ((args = lt.list()) != null) {
      // note, we ignore commands that we don't know about
      if (args.length < 2) continue;
	 
      /* Don't process arguments for other nodes */
      if (!node.equals(args[1])) continue;
	 
      if (args[0].equals("node")) {
	n = new Node(this, args[1], args[1]);
	n.setArgs(args, 2);
      } 
      else if (args[0].equals("channel")) {
	if (args.length < 3) throw new Exception("short channel");
	Channel c = Channel.New(this, args[2], args[2]);
	c.setArgs(args, 3);
	c.attachNode(n);
      } 
      else if (args[0].equals("application")) {
	if (args.length < 3) 
	    throw new Exception("short application");

	Class appClass = Class.forName(args[2]);
	Application a = (Application)(appClass.newInstance());    

	a.setBoss(this);
	a.setName(args[2]);
	a.setArgs(args, 3);
	a.attachNode(n);
      } 
      else if (args[0].equals("extension")) {
	if (args.length < 3) 
	    throw new Exception("short extension");

	Class extClass = Class.forName(args[2]);
	Extension e = (Extension)(extClass.newInstance());    

	e.setBoss(this);
	e.setName(args[2]);
	e.setArgs(args, 3);
	e.attachNode(n);
      } 
      else if (args[0].equals("manager")) {
	k.merge(args,2);
	setArgs(k);
      } 
      else if (args[0].equals("alternate")) {
	if (args.length < 5) throw new Exception("short alternate");
	n.addAlternate (new Alternate(args[2], 
				      (short)Integer.parseInt(args[3]),
				      ChannelAddress.New(args[4])));
      }
    }
      
    /* start ourselves, then rely on the node
       to start others in a safe order */
    if (n != null)
	n.start();
  }
  
  
  public static void main(String[] args) throws Exception {
    String use = "<file> <node> [keyargs]";
    String cl = "ants.ConfigurationManager";
    if (args.length < 2) {
      System.out.println("usage: " + cl + " " + use);
      System.exit(1);
    }
      
    new ConfigurationManager(args);
  }
   
}

