/* 
 *  $Id: DataCapsule.java,v 1.3 2000/04/02 21:30:37 andrew Exp $
 *  
 *  Copyright 1996 Massachusetts Institute of Technology
 * 
 *  Permission to use, copy, modify, distribute, and sell this software and
 *  its documentation for any purpose is hereby granted without fee, provided
 *  that the above copyright notice appear in all copies and that both that
 *  copyright notice and this permission notice appear in supporting
 *  documentation, and that the name of M.I.T. not be used in advertising or
 *  publicity pertaining to distribution of the software without specific,
 *  written prior permission.  M.I.T. makes no representations about the
 *  suitability of this software for any purpose.  It is provided "as is"
 *  without express or implied warranty.
 * 
 */


package ants;

/**
 * Data capsule processing, for transfer with default routing.
 *
 * @author David Wetherall
 */


public class DataCapsule extends Capsule {

  // WARNING: this is a builtin capsule, whose version number
  // must be manually kept in synch with  its functionality
  public static final String VERSION = "ants.DataCapsule 10/22/97";

  protected byte[] mid() { return findMID("ants.DataCapsule"); }
  protected byte[] gid() { return findGID("ants.DataCapsule");}    
  protected byte[] pid() { return findPID("ants.DataCapsule");}

  public short spt;
  public short dpt;

  // need to remove public instance variables above and use accessors
  // and mutators below ...

  public short getSrcPort() { return spt; }
  public short getDstPort() { return dpt; }
  public void setSrcPort(short p) { spt = p; }
  public void setDstPort(short p) { dpt = p; }

  private ByteArray data;
  
  public int length() {
    int s = Xdr.SHORT + Xdr.SHORT + Xdr.SHORT + 
      (data == null ? 0 : data.length());
    return super.length() + s;
  }
  
  public Xdr encode() {
    Xdr xdr = super.encode();
    xdr.PUT(spt);
    xdr.PUT(dpt);
    xdr.PUT(data);
    return xdr;
  }

  public Xdr decode() {
    Xdr xdr = super.decode();
    spt = xdr.SHORT();
    dpt = xdr.SHORT();
    data = xdr.BYTEARRAY();
    return xdr;
  }
  
  public ByteArray getData() {
    return data;
  }
  
  public void setData(ByteArray data) {
    this.data = data;
  }

  public boolean evaluate(Node n) {
    if (n.getAddress() ==  getDst()) {
      return n.deliverToApp(this, dpt);
    } else {
      return n.routeForNode(this, getDst());
    }
  }  
 
  public DataCapsule() { }

  public DataCapsule(short s, short d, int n, ByteArray p) {
    spt = s;
    dpt = d;
    setDst(n);
    data = p;
  }


  public static void main(String[] args) 
    throws Exception
  {
    Protocol dp = new DataProtocol();

    int addr = NodeAddress.fromString("1.2.3.4");
    byte[] pkt = { 1, 2, 3, 4, 5, 6, 7, 8 };

    DataCapsule f = new DataCapsule((short)1, (short)-1, addr, 
				    new ByteArray(pkt));

    // serialization
    Xdr xdr = new Xdr(f.length());
    f.xdr = xdr;
    f.encode();
    
//     if (l1 != f.length()) System.out.println("MISMATCH encode");

    // deserialization
    Class cl = Class.forName("ants.DataCapsule");
    DataCapsule t = (DataCapsule)(cl.newInstance());
    t.xdr = xdr;
    t.decode();

//     if (l2 != t.length()) System.out.println("MISMATCH decode");

    if (f.getDst() != t.getDst()) System.out.println("MISMATCH dst");
    if (f.spt != t.spt) System.out.println("MISMATCH spt");
    if (f.dpt != t.dpt) System.out.println("MISMATCH dpt");
    ByteArray fdata = f.getData();
    ByteArray tdata = t.getData();
    if (fdata.length() != tdata.length()) 
      System.out.println("MISMATCH length");
    for (int i = 0; i < fdata.length(); i++) 
      if (fdata.getByte(i) != tdata.getByte(i)) 
	System.out.println("MISMATCH data");
  }
}


 
