package ants;

/** Sends periodic keep-alive messages to the core node in order to 
 *  resubscribe to its address 
 *
 * @author Andrew Whitaker
*/

public final class KeepAliveThread extends Thread {

  protected Node parent;
  protected boolean halt;

  public KeepAliveThread (Node parent) {
    this.parent = parent;
    halt = false;
  }

  /** This can't be synchronized: we would deadlock */
  public void setHalt(boolean h) {
    halt = h;
  }

  /** We synchornize twice here because the RouteTable is already
      using the Node parent as a synchronization object */

  public void run () {            
    synchronized (this) {
      while (true) {
	try {
	  wait(DynamicAddressManager.KEEP_ALIVE_FREQUENCY);
	}
	catch (InterruptedException ie) {}

	if (halt) return;
	  
	DynamicAddressCapsule dac = new DynamicAddressCapsule 
	    (DynamicAddressCapsule.INIT, parent.link.source,
	     parent.damServer,parent.getAddress(),0);
	
	synchronized(parent) {
	  try {
	    dac.prime(0,255);
	    parent.context = dac;
	    dac.evaluate(parent);
	  }
	  catch (Exception exc) {
	    Entity.info("Exception occured during address keep alive" +
			exc);
	  }
	  finally {
	    parent.context = null;
	  }	
	}
      }
    }
  }
}
