/* 
 *  $Id: UDPChannel.java,v 1.1.1.1 2000/03/13 21:26:49 andrew Exp $
 *  
 *  Copyright 1996 Massachusetts Institute of Technology
 * 
 *  Permission to use, copy, modify, distribute, and sell this software and
 *  its documentation for any purpose is hereby granted without fee, provided
 *  that the above copyright notice appear in all copies and that both that
 *  copyright notice and this permission notice appear in supporting
 *  documentation, and that the name of M.I.T. not be used in advertising or
 *  publicity pertaining to distribution of the software without specific,
 *  written prior permission.  M.I.T. makes no representations about the
 *  suitability of this software for any purpose.  It is provided "as is"
 *  without express or implied warranty.
 * 
 */


package ants;

import java.net.*;
import java.io.*;

/**
 * UDP channel for nodes
 *
 * @author David Wetherall
 */

public class UDPChannel extends Channel {

  public static final int BUFSIZE = 1500;

  protected DatagramSocket socket;
  
  public int getMTU() { return BUFSIZE; }
      
  public boolean send(byte[] buf, int length, ChannelAddress dst) {
      
    if (! (dst instanceof UDPChannelAddress)) {
       Entity.warn("UDPChannel.send() passed non-UDP destination address");
       return false;
    }
    
    UDPChannelAddress a = (UDPChannelAddress)dst;
    DatagramPacket fr = new DatagramPacket(buf, length, a.address, a.port);
    
    try {
       socket.send(fr);
       if (LOGGING)
	   log(L[7], "transmitted " + fr.getLength() + " bytes");
    } 
    catch (IOException e) {
       return false;
    }
    
    
    return true;
  }
     
  public Capsule receive() throws Exception { 
    
    do {
       /* note: essential that we refresh p because of
	  the "ever smaller packets" receive screw */	  
       DatagramPacket p = new DatagramPacket(rxbuf.buf, rxbuf.len);
       
       try {
	  socket.receive(p);		  
	  
	  if (p.getLength() <= 0) {
	     if (LOGGING) log(L[7], "ignoring 0 byte read");
	     continue;
	  }
	  
	  if (LOGGING) log(L[7], "received " + p.getLength() + " bytes");
	  return decode(new Xdr(rxbuf, 0, p.getLength()));
	  
       }
       catch (IOException e) {	    
	  if (LOGGING) log(L[1], "DatagramSocket.receive() caught " + e);
	  continue;
       }
    } while (true);
  }
  
      
  public UDPChannel(Manager b, String n, UDPChannelAddress src)
    throws IOException {
     
     super(b, n, src);
     
     socket = new DatagramSocket(src.port);      
     txbuf = new Xdr(BUFSIZE);
     rxbuf = new Xdr(BUFSIZE);
  }
   
  public UDPChannel(Manager b, String n, String hp)throws Exception {
     this(b, n, new UDPChannelAddress(hp));
  }
  
}
 
