
/* 
 *  $Id: PongApplication.java,v 1.1.1.1 2000/03/13 21:26:49 andrew Exp $
 *  
 *  Copyright 1996 Massachusetts Institute of Technology
 * 
 *  Permission to use, copy, modify, distribute, and sell this software and
 *  its documentation for any purpose is hereby granted without fee, provided
 *  that the above copyright notice appear in all copies and that both that
 *  copyright notice and this permission notice appear in supporting
 *  documentation, and that the name of M.I.T. not be used in advertising or
 *  publicity pertaining to distribution of the software without specific,
 *  written prior permission.  M.I.T. makes no representations about the
 *  suitability of this software for any purpose.  It is provided "as is"
 *  without express or implied warranty.
 * 
 */


package apps;

import java.awt.*;
import java.awt.event.*;

import ants.*;
import utils.*;

/**
 * test GUI application with pong buttons
 *
 * @author David Wetherall, Mark Keaton
 * Modifed by Andrew Whitaker
 */

public class PongApplication extends Application 
    implements Runnable, ActionListener {

  final public static String[] defaults = {};
  
  int target;
  int sndCount = 0;
  int rcvCount = 0;
  long minRTT  = 100000000;
  long maxRTT  = 0;
  long sumRTT  = 0;
  Button ponger;
  Label minLabel;
  Label avgLabel;
  Label maxLabel;
  Label lossLabel;
  TextField iterations;
  TextField interval;
  Frame frame;
  
  synchronized public void receive(Capsule cap) {
    super.receive(cap);
    
    if (cap instanceof PongCapsule) {
      PongCapsule pcap = (PongCapsule)cap;
      
      if (pcap.loader) {
        return;
      }
      
      long rtt = (getNode().time() - pcap.timestamp);
      rcvCount++;
      sumRTT += rtt;
      
      if (rtt > maxRTT) {
        maxRTT = rtt;
        maxLabel.setText(Long.toString(maxRTT) + " ms");
      }
      
      if (rtt < minRTT) {
        minRTT = rtt;
        minLabel.setText(Long.toString(minRTT) + " ms");
      }
      
      avgLabel.setText(Double.toString((double)sumRTT / (double)rcvCount) +
                       " ms");
      lossLabel.setText(Double.toString((double)(sndCount - rcvCount) /
                                        (double)sndCount) + " %");
    }
  }
  

  public void actionPerformed (ActionEvent evt) {
    if (evt.getSource() == ponger) {
      new Thread(this).start();
    }
  }
  
  public void run(){
    int i;
    int iter   = Integer.parseInt(iterations.getText());
    int interv = Integer.parseInt(interval.getText());
    
    for (i = 0; i < 4; i++) {
      PongCapsule c = new PongCapsule(getNode().time(), true, port, port,
                                      target, null);
      send(c);
      getNode().sleep(100);
    }
    
    for (i = 0; i < iter; i++) {
      PongCapsule c = new PongCapsule(getNode().time(), false, port, port,
                                      target, null);
      sndCount++;
      send(c);
      getNode().sleep(interv);
    }
  }
  
  public void setArgs(KeyArgs k) throws Exception {
    k.merge(defaults);
    
    for (int i = 0; i < k.length(); i++) {
      if (k.key(i).equals("-target")){
        target = NodeAddress.fromString(k.arg(i));
        k.strike(i);
      }
    }
    
    super.setArgs(k);
  }
  
  public void start() throws Exception  {
    getNode().register(new PongProtocol());
    
    frame = new Frame ("Pong");
    
    // 2 rows by 2 columns with 2 pixel point spaces in between them
    frame.setLayout(new BorderLayout()); 
    
    ponger = new Button("ping " + NodeAddress.toString(target));
    ponger.addActionListener (this);
    
    Panel outputDisplay = new Panel();
    outputDisplay.setLayout(new GridLayout(8, 1, 1, 1));
    outputDisplay.add(new Label("Minimum Latency (ms)", Label.CENTER));
    minLabel = new Label("0", Label.CENTER);
    outputDisplay.add(minLabel);
    outputDisplay.add(new Label("Average Latency (ms)", Label.CENTER));
    avgLabel = new Label("0", Label.CENTER);
    outputDisplay.add(avgLabel);
    outputDisplay.add(new Label("Maximum Latency (ms)", Label.CENTER));
    maxLabel = new Label("0", Label.CENTER);
    outputDisplay.add(maxLabel);
    outputDisplay.add(new Label("Loss Rate (%)", Label.CENTER));
    lossLabel = new Label("0", Label.CENTER);
    outputDisplay.add(lossLabel);
    
    Panel iterDisplay = new Panel();
    iterDisplay.setLayout(new BorderLayout());
    iterDisplay.add("Center",
                    new Label("Num Iterations", Label.CENTER));
    iterations = new TextField("0");
    iterations.setEditable(true);
    iterDisplay.add("South", iterations);
    
    Panel intervalDisplay = new Panel();
    intervalDisplay.setLayout(new BorderLayout());
    intervalDisplay.add("Center",
                        new Label("Pong interval (in ms)", Label.CENTER));
    interval = new TextField("0");
    interval.setEditable(true);
    intervalDisplay.add("South", interval);
    
    Panel topPanel = new Panel(), botPanel = new Panel();
    topPanel.setLayout(new GridLayout(1, 2, 1, 1));
    botPanel.setLayout(new GridLayout(1, 2, 1, 1));
    
    topPanel.add(ponger);
    topPanel.add(outputDisplay);
    botPanel.add(iterDisplay);
    botPanel.add(intervalDisplay);
    
    frame.add("South", botPanel);
    frame.add("Center", topPanel);

    frame.addWindowListener (new WindowAdapter () {
      public void windowClosing (WindowEvent we) {
	System.out.println ("Quitting PongApplication...");
	System.exit(0);
      }
    });

    frame.pack();
    frame.show();
  }
  
  public PongApplication()  throws Exception {
    super();
  }
}
