
/* 
 *  $Id: PongCapsule.java,v 1.1.1.1 2000/03/13 21:26:49 andrew Exp $
 *  
 *  Copyright 1996 Massachusetts Institute of Technology
 * 
 *  Permission to use, copy, modify, distribute, and sell this software and
 *  its documentation for any purpose is hereby granted without fee, provided
 *  that the above copyright notice appear in all copies and that both that
 *  copyright notice and this permission notice appear in supporting
 *  documentation, and that the name of M.I.T. not be used in advertising or
 *  publicity pertaining to distribution of the software without specific,
 *  written prior permission.  M.I.T. makes no representations about the
 *  suitability of this software for any purpose.  It is provided "as is"
 *  without express or implied warranty.
 * 
 */


package apps;

import ants.*;


/**
 * pong capsule processing
 *
 * @author David Wetherall, Mark Keaton
 * modified by Andrew Whitaker
 */

public class PongCapsule extends DataCapsule {
  final private static byte[] MID = findMID("apps.PongCapsule");
  protected byte[] mid(){
    return(MID);
  }

  final private static byte[] PID = findPID("apps.PongCapsule");
  protected byte[] pid() {
    return(PID);
  }
  
  public long    timestamp = 0;
  public boolean pong      = false;
  public boolean loader    = false;
  
  public int length() {
    return(super.length() + Xdr.LONG + Xdr.BOOLEAN + Xdr.BOOLEAN);
  }
  
  public Xdr encode() {
    Xdr xdr = super.encode();
    xdr.PUT(timestamp);
    xdr.PUT(pong);
    xdr.PUT(loader);
    
    return(xdr);
  }
  
  public Xdr decode() {
    Xdr xdr = super.decode();
    timestamp = xdr.LONG();
    pong      = xdr.BOOLEAN();
    loader    = xdr.BOOLEAN();
    
    return(xdr);
  }

  
  public boolean evaluate(Node n) {
    if (n.getAddress() == getDst()) {
      pong = true;      
    }
    else {
      if (pong != true) {
        return(n.routeForNode(this, getDst()));
      }
    }
    
    if (n.getAddress() == getSrc()) {      
      return(n.deliverToApp(this, dpt));
    }
    else {
      if (pong) {
        return(n.routeForNode(this, getSrc()));
      }
    }
    
    return(false);
  }
  
  public PongCapsule() { }

  public PongCapsule(long ts, boolean l, short sa, short da, int na,
                     ByteArray d) {
    super(sa, da, na, d);
    this.timestamp = ts;
    this.loader    = l;
  }

}
