package apps;

/** This is a very simple traceroute application.  A capsule is routed from 
    the source to the destination, recording each hop along the way.  The 
    application waits three seconds for the capsule to return.

    @author Andrew Whitaker
*/

import java.awt.*;
import java.awt.event.*;
import java.util.Hashtable;

import ants.*;
import utils.*;

public class TraceApplication extends Application 
    implements ActionListener {
    
  private TextArea area;
  private TextField field;

  // A table of traces that haven't yet returned 
  private Hashtable outstandingTraces;

  synchronized public void receive (Capsule cap) {
    super.receive(cap);
    
    if (cap instanceof TraceCapsule) {
      TraceCapsule tc = (TraceCapsule) cap;
      outstandingTraces.remove(new Long(tc.getTimeStamp()));
      
      long rtt = getNode().time() - tc.getTimeStamp();
      int[] path = tc.getPath();

      addText("\nRoute received!\nRTT: " + rtt + "\n");
      for (int i=0;i< (path.length-1);i++) 
	addText (NodeAddress.toString(path[i]) + ", ");
      addText(NodeAddress.toString(path[path.length-1]) + "\n");
    }
  }

  /* Send out a trace capsule */
  public void actionPerformed(ActionEvent ae) {

    try {
      String dest = field.getText().trim();
      int target = NodeAddress.fromString(dest);
      final long time = getNode().time();
      
      Capsule capsule = new TraceCapsule(port,port,target,time);
      send(capsule);

      outstandingTraces.put(new Long(time),new Object());

      /* Wait 3 seconds for the trace to return */
      new Thread () {
	public void run() {
	  try {
	    Thread.sleep(3000);
	  }
	  catch (InterruptedException ie) {}

	  Object o = outstandingTraces.get(new Long (time));
	  if (o != null)
	    addText("\n#### No route found #### \n");
	}
      }.run();
	      	  	  
    }
    catch (Exception exc) {
      exc.printStackTrace();
      addText("\n##### Invalid address #####\n");
    }
  }

  public void addText (String s) {
    area.append (s);
  }
    
  public void start()  throws Exception {
    getNode().register(new TraceProtocol());
    outstandingTraces = new Hashtable();

    Frame f = new Frame("Trace Route");
    f.addWindowListener (new WindowAdapter () {
      public void windowClosing (WindowEvent we) {
	System.exit(0);
      }
    });


    area = new TextArea("",8,20,TextArea.SCROLLBARS_VERTICAL_ONLY);
    field = new TextField (15);
    field.addActionListener (this);

    Button button = new Button("Start");
    button.setActionCommand ("start");
    button.addActionListener (this);

    Panel north = new Panel();
    north.add(new Label ("Destination:"));
    north.add(field);
    north.add(button);

    f.setLayout(new BorderLayout());
    f.add(north,BorderLayout.NORTH);
    f.add(area,BorderLayout.CENTER);

    f.pack();
    f.setVisible(true);
  }

}
    

    
