package apps;

import ants.*;

public class TraceCapsule extends DataCapsule {
   
  
  protected boolean returning;
  protected long timeStamp;
  protected int[] path;

  public int length() {
    return (super.length() + Xdr.BOOLEAN + Xdr.LONG + Xdr.INT + 
	    (path.length * Xdr.INT));
  }

  public boolean isReturning () {
    return returning;
  }

  public long getTimeStamp() {
    return timeStamp;
  }

  public int[] getPath() {
    return path;
  }

  public Xdr encode() {
    Xdr xdr = super.encode();

    xdr.PUT(timeStamp);
    xdr.PUT(returning);
    xdr.PUT(path.length);
    
    for (int i=0;i<path.length;i++) 
      xdr.PUT (path[i]);
    
    return xdr;
  }

  public Xdr decode () {
    Xdr xdr = super.decode();

    timeStamp = xdr.LONG();
    returning = xdr.BOOLEAN();
    
    int hops = xdr.INT();
    path = new int[hops];

    for (int i=0;i<hops;i++) 
      path[i] = xdr.INT();

    return xdr;
  }

  protected byte [] mid() {
    return findMID("apps.TraceCapsule");
  }
  
  protected byte [] pid() {
    return findPID ("apps.TraceCapsule");
  }
  
  protected byte[] gid() {
    return findGID ("apps.TraceCapsule");
  }

  protected void setReturning (boolean r) {
    returning = r;
  }

  protected void addHop (int address) {

    int [] newPath = new int[path.length + 1];
    for (int i=0;i<path.length;i++)
      newPath[i] = path[i];
    newPath[path.length] = address;

    path = newPath;
  }

  public boolean evaluate (Node n) {
    if (! returning) {
      addHop (n.getAddress());
      
      if (n.getAddress() == getDst()) {
	returning = true;
	return (n.routeForNode(this,getSrc()));
      }
      else {
	return (n.routeForNode(this,getDst()));
      }
    }

    /* OK, we're on the way back... */

    else if (n.getAddress() == getSrc()) {
      return(n.deliverToApp(this,getSrcPort()));
    }
    else {
      return (n.routeForNode(this,getSrc()));
    }
  }
	
  public TraceCapsule() {}

  public TraceCapsule (short sa,short da,int na,long ts) {
    super (sa,da,na,null);
    path = new int[0];
    returning = false;
    timeStamp = ts;
  }
}
