/* 
 *  $Id: ClassCode.java,v 1.2 2000/03/31 00:10:31 andrew Exp $
 *  
 *  Copyright 1996 Massachusetts Institute of Technology
 * 
 *  Permission to use, copy, modify, distribute, and sell this software and
 *  its documentation for any purpose is hereby granted without fee, provided
 *  that the above copyright notice appear in all copies and that both that
 *  copyright notice and this permission notice appear in supporting
 *  documentation, and that the name of M.I.T. not be used in advertising or
 *  publicity pertaining to distribution of the software without specific,
 *  written prior permission.  M.I.T. makes no representations about the
 *  suitability of this software for any purpose.  It is provided "as is"
 *  without express or implied warranty.
 * 
 */


package utils;

import java.io.*;

// For some reason, this import is needed by anetd
import java.lang.*;
import java.net.*;

import ants.*;

/**
 * Obtain classfile for a class
 *
 * @author David Wetherall
 */


public class ClassCode {

    /**
   * Convert a fully qualified class name into an array
   * of its bytes that is the classfile. Search the standard
   * classpath and load the file -- can't find any other way
   */

   public static byte[] get(String cn) throws Exception {
      // Add a terminating ':' on to the path so that we 
      // are sure we search the last element
      String cp = System.getProperty("java.class.path") + File.pathSeparator;
      String ce = File.separator + cn.replace('.', File.separatorChar) + ".class";
      FileInputStream cfi = null;
      File cf = null;
      int last = 0;
      int pos;
      
      // Added by Livio Ricciulli to enable ANTS to run through Anetd.

      URLConnection connection;
      BufferedInputStream in;     
      
      while ((pos = cp.indexOf(File.pathSeparatorChar, last)) > 0) {
	  cf = new File(cp.substring(last, pos) + ce);
	  last = pos+1;
	  try {
	      cfi = new FileInputStream(cf);
	      break;
	  } 
	  catch (FileNotFoundException e) {
	      continue;
	  }
      }
      
      
      if (ConfigurationManager.USE_ANETD) {
	  // Changes by Livio Ricciulli to enable ANTS to run through Anetd.
	  
	  // Further modifications were made in this section to ensure that 
	  // entire packets are downloaded.
	  
	  if (cf == null || cfi == null) {
	      
	      URL u = new URL(System.getProperty("codebase") +
			      cn.replace('.', File.separatorChar) + ".class");
	      connection = u.openConnection();
	      int len    = connection.getContentLength();
	      if (len < 1) {
		  throw new Exception("url file length unavailable for " + cn);
	      }
	      
	      in        = new BufferedInputStream(connection.getInputStream());
	      byte[] cb = new byte[len];
	      int bytes = 0;
	      int rv    = 0;
	      
	      while (bytes < len) {
		  rv = in.read(cb, bytes, (len - bytes));
		  if (rv >= 0) {
		      bytes += rv;
		  }
	      }
	      in.close();
	      return cb;
	  }
	  else {
	      int len = (int)cf.length();
	      byte[] cb = new byte[len];
	      if (cfi.read(cb) != len) throw new Exception("short read for "+ cn);
	      return cb;
	  } 
      }
      else {
	  if (cf == null || cfi == null)
	      throw new Exception("code not found for "+ cn);
	  
	  int len = (int)cf.length();
	  byte[] cb = new byte[len];
	  
	  if (cfi.read(cb) != len)
	      throw new Exception("short read for "+ cn);
	  
	  return cb;
      }
   }      
   

   /** Test the class. */
   public static void main(String[] args) throws Exception {
      byte[] code = get("utils.ClassCode");
   }
}

